#!/bin/sh
# the next line restarts using wish \
    exec tclsh "$0" "$@"

# HtmlAlbum - Gui for htmlalbum XML backend.

# $Id: albumgui.tcl,v 1.2 2004/09/21 11:30:51 davidw Exp $

package require Tk

package provide albumgui 0.1

package require BWidget

set auto_path [linsert $auto_path end [file dir [info script]]]

package require album
package require manager
package require gridinfo
package require magick
package require preferencesgui
package require progressbar

set scriptdir [file normalize [file dir [info script]]]

if { $tcl_platform(platform) == "unix" } {
    source [file join [file dir [info script]] tkfbox.tcl]
}

namespace eval album::gui {
    set Album {}
    namespace import ::gridinfo::*
    variable prefsmenu 0
    variable busy 0
}

# album::gui::init --
#
#	Set up the application with a menu.
#
# Side Effects:
#	Sets up the File and Help menus.

proc album::gui::init {} {
    variable scriptdir
    variable Album
    variable app

    catch { wm withdraw . }
    set Album [namespace current]::[Album \#auto]
    set app [toplevel .htmlalbum]

    wm title $app "HTMLALbum"
    menu $app.mbar
    $app config -menu $app.mbar

    $app.mbar add cascade -label "File" -underline 0 \
	-menu [menu $app.mbar.file -tearoff 0]

    $app.mbar add cascade -label "Help" -underline 0 \
	-menu [menu $app.mbar.help -tearoff 0]

    # File menu

#    set logo [magickimage [file join $::scriptdir logo.jpg]]
#    set logo [image create photo -file [file join $::scriptdir logo.jpg]]
#    label $app.logo -image $logo
#    grid $app.logo -sticky ewns

    set m $app.mbar.file
#    $m add command -label "New Album" -underline 0 \
#	-command album::gui::newalbum
    $m add command -label "Open Album" -underline 0 \
	-command album::gui::openalbum
    $m add command -label "Add Files" -underline 0 \
	-command album::gui::addfiles
    $m add command -label "Save Album" -underline 0 \
	-command album::gui::savealbum
    $m add command -label "Generate HTML Photo Album" -underline 0 \
	-command album::gui::savehtmlfiles
    $m add command -label "Exit" -underline 1 \
	-command album::gui::quit

    # Help menu
    set m $app.mbar.help
    $m add command -label "Help" -underline 0 -command album::gui::help
    $m add command -label "About" -underline 0 -command album::gui::about
    unset m

    displayfiles
}

proc album::gui::newalbum {} {
    variable Manager

    set fl [tk_chooseDirectory]
    if { $fl == "" } {
	return
    }

    set Manager [namespace current]::[Manager \#auto -directory $fl]
    album::gui::managemenu
}

proc album::gui::managemenu {} {
    variable Manager
    variable ManagerMenu 0

    if { $ManagerMenu == 0 } {
	$app.mbar add command -label "Manage Albums" -underline 0 \
	    -command [list album::gui::manage::create $Manager]
	set ManagerMenu 1
    }
}


proc album::gui::openalbum {} {
    variable Album
    variable mt

    set types {
	{"XML" .xml}
	{"All files" *}
    }
    set fl [tk_getOpenFile -filetypes $types -initialfile [glob -nocomplain htmlalbum.xml]]
    if { $fl == "" } {
	return
    }

    cd [file dirname $fl]
    $Album getxml $fl
    $mt.etitle delete 0 end
    $mt.etitle insert 0 [$Album cget -title]
    album::gui::files2screen
}


# album::gui::loadxmlfile --
#
#	Interface to htmlalbum's loadxml file command.  Selecting
#	which file to load is currently not supported.
#
# Side Effects:
#	Loads and parses the XML file, via htmlalbum, and then calls
#	displayfiles to set up the main window.
#
# Results:
#	None.

proc album::gui::loadxmlfile {newalbum} {
    variable Album
#    set types {
#	{"XML" .xml}
#	{"All files" *}
#    }
#    set fl [tk_getOpenFile -filetypes $types -initialfile [glob -nocomplain htmlalbum.xml]]
#    if { $fl == "" } {
#	return
#    }
    if { [catch {
#	set dirname [file dirname $fl]
#	if { $dirname == "." } { set dirname [pwd] }
	set Album $newalbum
#	$Album setdirectory $dirname
#	$Album getxml $fl
    } err] } {
	tk_messageBox -type ok -message "There were errors parsing '$fl' - please select another file and try again"
	puts stderr "$::errorInfo"
	return
    }

    album::gui::displayfiles
}

# album::gui::savealbum --
#
#	Saves the XML file describing the contents of the photo album.

proc album::gui::savealbum {} {
    variable Album
    variable mt

    if { [$Album cget -xmlfile] == "" } {
	set types {
	    {"XML" .xml}
	    {"All files" *}
	}
	set fl [tk_getSaveFile -filetypes $types -initialfile htmlalbum.xml]

	if { $fl == "" } {
	    return
	}
	$Album configure -xmlfile $fl
    }

    album::gui::saveseldata $album::gui::FileSel
    $Album configure -title [$mt.etitle get]
    $Album writexml
}

# album::gui::savehtmlfiles --
#
#	Save HTML files to disk, creating the actual photo album.


proc album::gui::savehtmlfiles {} {
    variable Album
    variable mt

    progressbar::create "Creating HTML Photo Album"

    album::gui::saveseldata $album::gui::FileSel
    $Album configure -title [$mt.etitle get]
    $Album writehtml index.html progressbar::callback
    progressbar::done
}

# album::gui::addfiles --
#
#	Examines the files in the current directory, and generates
#	information about them, to serve as the basis for an Album.
#	TODO: let the user select which files to use.
#
# Side Effects:
#	Set up the display with the information gathered.

proc album::gui::addfiles {} {
    variable Album

    set types {
	{"JPEG" .jpg}
	{"JPEG" .jpeg}
	{"GIF" .gif}
	{"PNG" .png}
	{"All files" *}
    }

    set fl [tk_getOpenFile -filetypes $types -multiple 1 \
		-initialfile [glob -nocomplain *.jpg] \
		-initialdir [pwd]]

    if { $fl == "" } {
	return
    }

    foreach f $fl {
	lappend files [file tail $f]
    }

    if { [catch {

	$Album getfiles $files
    } err] } {
	tk_messageBox -type ok -message "There was an error generating an album from files: $err"
	puts stderr "$::errorInfo"
	return
    }
    album::gui::displayfiles
}


# album::gui::quit --
#
#	Quit the application after asking a confirmation.  TODO: check
#	and see if the data has been saved to disk.

proc album::gui::quit {} {
    set a [tk_messageBox -type yesno -message "Are you sure you want to quit?"]
    if {[string equal $a "yes"]} {
	exit
    }
}


# album::gui::help --
#
#	Display the help text.

proc album::gui::help {} {
    variable app
    catch {
	destroy $app.help
    }
    toplevel $app.help
    set h [text $app.help.h  -yscrollcommand [list $app.help.sc set]]
    set sc [scrollbar $app.help.sc -command [list $app.help.h yview]]
    set b [button $app.help.b -text Dismiss -command [list wm withdraw $app.help]]
    pack $b -fill x -side bottom
    pack $h -fill both -side left -expand 1
    pack $sc -side right  -expand 1 -fill y
    $h tag config title -justify center -font {Helvetica -20 bold}
    $h tag config bold -font {Helvetica -12 bold}
    $h tag config right -font {Helvetica -12} -wrap word
    $h insert 0.0 "HTMLAlbum Help\n\n" title \
	"\nOpen XML File - " bold "This command loads an XML file that contains information about the album." right \
	"\n\nCreate Album from Files - " bold  "Selecting this will start a new photo album with all the files matching *.jpg in the current directory." right \
	"\n\nSave XML File - " bold  "Saves the htmlalbum.xml file, using the data currently in HTMLAlbum." right \
	"\n\nGenerate HTML Photo Album - " bold  "Creates the actual HTML files that constitute the photo album." right \
	"\n\n - " bold  "Creates the actual HTML files that constitute the photo album." right
    $h config -state disabled
}

# album::gui::about --
#
#	About HTMLAlbum

proc album::gui::about {} {
    variable app
    catch { destroy $app.about }
    set a [toplevel $app.about -class dialog]
    wm title $a "About HTMLAlbum"
    button $a.b -text Dismiss -command [list wm withdraw $a]


#    text $a.text -height 9 -bd 1 -width 60
    text $a.text -height 9 -width 60
    pack $a.b -fill x -side bottom
    pack $a.text -fill both -side left -expand 1
    $a.text tag config center -justify center
    $a.text tag config title -justify center -font {Courier -18 bold}
    $a.text insert 1.0 "About HTMLAlbum" title \
	"\n\nCopyright (C) 2000-2003 David N. Welton <davidw@dedasys.com>" \
	center \
	"\n\nAvailable from http://www.dedasys.com/" center
    $a.text config -state disabled
}


# album::gui::Button --
#
#	Handles <Button> events in the file listbox.
#
# Arguments:
#	w - widget.
#	y - y position of event.
#
# Side Effects:
#	Saves the location of the listbox item selected in the FileSel
#	variable.
#
# Results:
#	None.

proc album::gui::Button {w x y} {
    variable FileSel

    variable busy
    if { $busy == 1 } {
	return
    }
    set busy 1

    if { [info exists FileSel] } {
	album::gui::saveseldata $FileSel
    }
    $w itemconfigure $FileSel -bg {}
    $w itemconfigure $FileSel -fg black
    set FileSel [$w nearest $y]
    $w itemconfigure $FileSel -bg #4a6984
    $w itemconfigure $FileSel -fg white
    album::gui::getselinfo [$w index @$x,$y]
    set busy 0
}


# album::gui::Up --
#
#	Move album up in listbox.
#
# Arguments:
#	w - widget.
#
# Side Effects:
#	Changes the order of the images in the album.
#
# Results:
#	None.

proc album::gui::Up {w} {
    variable Album
    variable FileSel
    if { $FileSel == 0 } {
	return
    }
    set oldtxt [$w get $FileSel]
    set oldSel $FileSel
    incr FileSel -1
    $Album move $oldSel $FileSel


    $w insert $FileSel $oldtxt
    incr oldSel
    $w delete $oldSel

    $w selection clear 0 end
    $w selection set $FileSel
}

# album::gui::Down --
#
#	Move entry down in the listbox.
#
# Arguments:
#	w - widget.
#
# Side Effects:
#	Changes the order of the images in the album.
#
# Results:
#	None.

proc album::gui::Down {w} {
    variable FileSel
    variable Album

    set last [$w size]
    incr last -1
    if { $FileSel == $last } {
	return
    }
    set oldtxt [$w get $FileSel]
    set oldSel $FileSel
    incr FileSel
    $Album move $oldSel $FileSel

    $w delete $oldSel
    $w insert $FileSel $oldtxt

    $w selection clear 0 end
    $w selection set $FileSel
}


# album::gui::ButtonRelease --
#
#	Procedure called when the mouse button is released.
#
# Arguments:
#	w - widget.
#	x - x event coordinate.
#	y - y event coordinate.
#
# Side Effects:
#	Sets FileSel variable.
#
# Results:
#	None.

proc album::gui::ButtonRelease {w x y} {
    variable Album
    variable FileSel

    set To [$w curselection]
    set oldtxt [$w get $FileSel]
    set oldbg [$w itemcget $FileSel -bg]
    set oldfg [$w itemcget $FileSel -fg]
    $w itemconfigure $FileSel -bg red
    $Album move $FileSel $To
    if { $FileSel > $To } {
	$w delete $FileSel
	$w insert $To $oldtxt
	$w itemconfigure $To -bg $oldbg
	$w itemconfigure $To -fg $oldfg
    } else {
	incr To
	$w insert $To $oldtxt
	$w itemconfigure $To -bg $oldbg
	$w itemconfigure $To -fg $oldfg
	$w delete $FileSel
    }
    $w selection clear 0 end
    $w selection set @$x,$y
    album::gui::getselinfo [$w index @$x,$y]
    set FileSel [$w curselection]
}


# album::gui::displayfiles --
#
#	This is the big one.  It creates the display and populates it
#	with information about the files.
#
# Arguments:
#	None.
#
# Side Effects:
#	Sets up a variety of frames: one for the listbox and
#	scrollbar, one for the image and scrollbars, and then one for
#	the filename, title, and caption.
#
# Results:
#	None.

proc album::gui::displayfiles {} {
    #$app. configure ; #-bg black
    variable Album
    # The main frame.
    variable scriptdir
    variable m
    # The file listbox (upper left).
    variable mf
    # The file information (lower panel).
    variable fi
    # The image (upper right).
    variable im
    # The album title (top).
    variable mt
    variable mfl
    variable CanvasPhoto
    variable ToolBar

    variable app

    catch {
	destroy $app.main
    }
    catch {
	grid forget $app.logo
	destroy $app.logo
    }

    set m [frame $app.main] ; #-bg black
    $app.mbar add command -label "Preferences" -underline 0 \
	-command album::gui::prefs
    set prefsmenu 1
    $m configure -padx 0
    $m configure -pady 0
    grid $m -sticky news
    panedwindow $app.main.top
    set ToolBar [frame $app.main.toolbar] ;#-bg black
    $ToolBar configure -padx 0
    $ToolBar configure -pady 0

    set mt [frame $app.main.title -relief groove] ;#-bg black
    set mf [frame $app.main.top.files -relief groove] ;#-bg black
    set im [frame $app.main.top.image -relief groove] ;#-bg black

    $app.main.top add $app.main.top.files $app.main.top.image
    $app.main.top paneconfigure $app.main.top.image -sticky news
    set fi [frame $app.main.fileinfo] ;#-bg black

    grid $ToolBar -sticky w -columnspan 2
    grid $mt -sticky news -columnspan 2
    grid $app.main.top -sticky news
#    grid $mf $im -sticky news
    grid $fi -sticky news -columnspan 2

    grid columnconfigure $app [column $m] -weight 1
    grid rowconfigure $app [row $m] -weight 1
    grid rowconfigure $m [row $app.main.top] -weight 1
    grid columnconfigure $m [column $mt] -weight 1
    grid columnconfigure $m [column $app.main.top] -weight 1
 #   grid rowconfigure $m [row $im] -weight 6
 #   grid rowconfigure $m [row $fi] -weight 1
 #   grid columnconfigure $m [column $mf] -weight 1
 #   grid columnconfigure $m [column $im] -weight 4

    canvas $im.img -xscrollcommand [list $im.xscr set] \
	-yscrollcommand [list $im.yscr set]
    scrollbar $im.yscr -command [list $im.img yview] -takefocus 0
    scrollbar $im.xscr  -orient horiz -command [list $im.img xview] -takefocus 0
    # Save image number for later manipulation
    set CanvasPhoto [$im.img create image 0 0 -anchor nw]
    grid $im.xscr -sticky news
    grid $im.img $im.yscr -sticky news

    set mfbuttons [frame $mf.buttons]
    set mfup [button $mfbuttons.up -text "Up" -command [list album::gui::Up $mf.files]]
    set mfdown [button $mfbuttons.down -text "Down" -command [list album::gui::Down $mf.files]]
    set mfscroll [scrollbar $mf.filescroll -command [list $mf.files yview]  -takefocus 0]
    set mfl [listbox $mf.files -yscrollcommand [list $mf.filescroll set]]

    bind $mfl <Button> [list album::gui::Button %W %x %y]
    # bind $mfl <ButtonRelease> [list album::gui::ButtonRelease %W %x %y]

    grid $mfup $mfdown
    grid configure $mfup -sticky news
    grid configure $mfdown -sticky news
    grid $mfbuttons -sticky ew -columnspan 2
    grid $mfl $mfscroll

    grid configure $mfl -sticky news
    grid configure $mfscroll -sticky ns

    # Toolbar.
    button $ToolBar.new -text "Add Image" -compound left -image \
	[magickimage [file join $::scriptdir addimage36.png]] \
	-command album::gui::addimage

    button $ToolBar.del -text "Remove Image" -compound left -image \
	[magickimage [file join $::scriptdir delimage36.png]] \
	-command album::gui::removeimage

    button $ToolBar.edit -text "Edit Image" -command album::gui::editimage
    button $ToolBar.shrink -text "Shrink Image %50" -command album::gui::shrinkimage
    button $ToolBar.rotate -text "Rotate Image 90 Degrees" -command album::gui::rotateimage

    button $ToolBar.makealbum -text "Make HTML Album" -compound left -image \
	[magickimage [file join $::scriptdir makealbum36.png]] \
	-command album::gui::savehtmlfiles

    grid $ToolBar.new $ToolBar.del $ToolBar.edit $ToolBar.shrink $ToolBar.rotate $ToolBar.makealbum -sticky w
#    grid rowconfigure $ToolBar [row $ToolBar.new] -weight 1
#    grid columnconfigure $ToolBar [column $ToolBar.new] -weight 1
#    grid columnconfigure $ToolBar [column $ToolBar.del] -weight 1

    # Title / directory.

#    set titles [$Manager titles]
    set titles FIXME
    set titleidx [lsearch -exact $titles [$Album cget -title]]
#    set albumscombo [ComboBox $mt.tcombo -values $titles]
#    $albumscombo setvalue @$titleidx
    label $mt.ltitle -text "Album Title:" -anchor w
    entry $mt.etitle -width 25
#    label $mt.ldir -text "Directory:" -anchor w
#    label $mt.rdir -text [$Album setdirectory]
    $mt.etitle insert 0 [$Album cget -title]

    label $fi.fnamel -text "Filename:" -anchor w
    entry $fi.fnamer

    label $fi.fsizel -text "File Size:" -anchor w
    label $fi.fsizer -text "" -anchor w

    label $fi.title -text "Title:" -anchor w
    entry $fi.etitle

    label $fi.lcaption -text "Caption:" -anchor w
    text $fi.ecaption -height 3

    label $fi.keywords -text "Comma separated\nlist of keywords:" -justify left
    entry $fi.ekeywords


#    grid $albumscombo -sticky nws
    grid $mt.ltitle $mt.etitle
    grid configure $mt.ltitle $mt.etitle -sticky w
#    grid configure $mt.ldir $mt.rdir -sticky e
    grid columnconfigure $mt [column $mt.ltitle] -weight 0
#    grid columnconfigure $mt [column $mt.ldir] -weight 1
#    grid columnconfigure $mt [column $mt.tcombo] -weight 1
#    grid columnconfigure $mt [column $mt.ldir] -weight 1

    grid $fi.fnamel $fi.fnamer $fi.fsizel $fi.fsizer -sticky nswe

    grid $fi.title $fi.etitle -sticky nswe
    grid configure $fi.etitle -columnspan 3
    grid $fi.lcaption $fi.ecaption -sticky nswe
    grid configure $fi.ecaption -columnspan 3
    grid $fi.keywords $fi.ekeywords -sticky news
    grid configure $fi.ekeywords -columnspan 3

   # grid columnconfigure $mt [column $fi.etitle] -weight 1

    grid columnconfigure $mf [column $mfl] -weight 1
    grid rowconfigure $mf [row $mfl] -weight 4

    grid rowconfigure $mfbuttons [row $mfup] -weight 1
    grid columnconfigure $mfbuttons [column $mfup] -weight 1
    grid columnconfigure $mfbuttons [column $mfdown] -weight 1

    grid columnconfigure $fi [column $fi.fnamer] -weight 1 ;#-pad 10
    grid rowconfigure $fi [row $fi.ecaption] -weight 2 ;#-pad 10

    grid rowconfigure $im [row $im.img] -weight 1
    grid columnconfigure $im [column $im.img] -weight 1

    album::gui::files2screen
}

proc album::gui::files2screen {} {
    variable Album
    variable mfl

    foreach fn [$Album allcget -htmlfile] {
	$mfl insert end [file rootname [file tail $fn]]
    }
    $mfl selection set 0
    album::gui::getselinfo 0
    set ::album::gui::FileSel 0
}


# album::gui::saveseldata --
#
#	Save data from last selection.  When the user changes the
#	selected image, it is necessary to save the old data to the
#	internal representation.
#
# Arguments:
#	idx - the index of the data to save.

proc album::gui::saveseldata {idx} {
    variable Album
    variable fi
    variable mfl

    if { [$Album empty] } {
	return
    }

    set title [string trim [$fi.etitle get]]
    set caption [string trim [$fi.ecaption get 0.0 end]]
    set keywords [split [string trim [$fi.ekeywords get]] ","]
    set filename [string trim [$fi.fnamer get]]

    set fileroot [file root $filename]
    $mfl delete $idx
    $mfl insert $idx $fileroot

    $Album imageconfigure $idx -filename $filename
    if { $title != "" } {
	$Album imageconfigure $idx -title $title
    }
    $Album imageconfigure $idx -caption $caption
    $Album imageconfigure $idx -keywords $keywords
}


# album::gui::getselinfo --
#
#	Fills in the filename, title, caption and keywords fields, as
#	well as updating the image displayed to that selected by the
#	user.
#
# Arguments:
#	idx - the index of the image and accompanying data to use.

proc album::gui::getselinfo {idx} {
    variable fi
    variable im
    variable Photo
    variable CanvasPhoto
    variable firsttime
    variable Album
    variable app

    if { [$Album empty] } {
	return
    }

    set oldcursor [. cget -cursor]
    $app configure -cursor watch
    update
    $fi.fnamer delete 0 end
    $fi.fnamer insert 0 [$Album imagecget $idx -filename]

    $fi.fsizer configure -text [file size [$Album imagecget $idx -filename]]
    $fi.etitle delete 0 end
    $fi.etitle insert 0 [$Album imagecget $idx -title]
    $fi.ecaption delete 0.0 end
    $fi.ecaption insert 0.0 [$Album imagecget $idx -caption]
    $fi.ekeywords delete 0 end
    $fi.ekeywords insert 0 [join [$Album imagecget $idx -keywords] ", "]
    # A cache of N photos might not be a bad idea.
    catch { image delete $Photo }
    set Photo [magickimage [$Album imagecget $idx -filename]]
    $im.img configure -scrollregion [list 0 0 [image width $Photo] \
					 [image height $Photo]]
    # All this is to center the image.
    # Only do it the first time.  After that, the user's will is
    # obeyed.
    if { ! [info exists firsttime] } {
	set firsttime 1

	set left [$im.img xview]
	set right [lindex $left 1]
	set left [lindex $left 0]

	set top [$im.img yview]
	set bottom [lindex $top 1]
	set top [lindex $top 0]

	update ;# Otherwise the sizes aren't updated.

	$im.img xview moveto [expr {(1 - double($right))  / 2.0}]
	$im.img yview moveto [expr {(1 - double($bottom)) / 2.0}]
    }
    $im.img itemconfigure $CanvasPhoto -image $Photo
    $app configure -cursor $oldcursor
}

# album::gui::removeimage --
#
#	Remove an image from the album.
#
# Arguments:
#	None.
#
# Side Effects:
#	Removes the image from the list, and selects another image.
#
# Results:
#	None.

proc album::gui::removeimage {} {
    variable Album
    variable FileSel
    variable mfl

    set size [$mfl size]
    if { $size == 0 } {
	return
    }

    $Album remove $FileSel
    $mfl delete $FileSel
    if { $size - 1 == $FileSel } {
	incr FileSel -1
    }
    $mfl selection set $FileSel
    album::gui::getselinfo $FileSel
}

proc album::gui::editimage {} {
    variable mfl
    variable FileSel
    variable Album
#    variable im
#    variable Photo
#    variable CanvasPhoto

    set fn [$Album imagecget $FileSel -filename]
    exec gimp $fn &
}



proc album::gui::shrinkimage {} {
    variable mfl
    variable FileSel
    variable Album
    variable im
    variable Photo
    variable CanvasPhoto

    set a [tk_messageBox -type yesno -message "Are you sure you want to shrink the image?"]
    if {! [string equal $a "yes"]} {
	return
    }

    set fn [$Album imagecget $FileSel -filename]
    set magimg [magick create wand]
    $magimg ReadImage $fn
    $magimg minify
    $magimg WriteImage $fn
    magick delete $magimg

    catch { image delete $Photo }
    set Photo [magickimage [$Album imagecget $FileSel -filename]]
    $im.img configure -scrollregion [list 0 0 [image width $Photo] \
					 [image height $Photo]]
    $im.img itemconfigure $CanvasPhoto -image $Photo
}

proc album::gui::rotateimage {} {
    variable mfl
    variable FileSel
    variable Album
    variable im
    variable Photo
    variable CanvasPhoto

    set a [tk_messageBox -type yesno -message "Are you sure you want to rotate the image?"]
    if {! [string equal $a "yes"]} {
	return
    }

    set fn [$Album imagecget $FileSel -filename]
    set magimg [magick create wand]
    $magimg ReadImage $fn
    set black [magick create pixel]
    $black set red 0 green 0 blue 0
    $magimg rotate $black 90
    magick delete $black
    $magimg WriteImage $fn
    magick delete $magimg

    catch { image delete $Photo }
    set Photo [magickimage [$Album imagecget $FileSel -filename]]
    $im.img configure -scrollregion [list 0 0 [image width $Photo] \
					 [image height $Photo]]
    $im.img itemconfigure $CanvasPhoto -image $Photo
}



# album::gui::addimage --
#
#	Add an image to the end of the list.
#
# Arguments:
#	None.
#
# Side Effects:
#	Adds an image to the list, and selects it.
#
# Results:
#	None.

proc album::gui::addimage {} {
    variable mfl
    variable FileSel
    variable Album

    album::gui::saveseldata $FileSel

    set types {
	{"JPEG" .jpg}
	{"JPEG" .jpeg}
	{"GIF"  .gif}
	{"PNG"  .png}
	{"All files" *}
    }
    set fl [tk_getOpenFile -filetypes $types]
    if { $fl == "" } {
	return
    }
    set filename [file tail $fl]
    $Album newimage $filename
    $mfl insert end [file rootname $filename]
    set size [$mfl size]
    incr size -1
    #puts "Inserting [file tail $fl] [file rootname $filename] $size"
    $mfl selection set $size
    set FileSel $size
    album::gui::getselinfo $size
}

# album::gui::prefs --
#
#	Open the preferences dialog.

proc album::gui::prefs {} {
    variable prefs
    variable Album
    prefs::create album::gui::saveprefs
    prefs::addwidget "Index template file:" file \
	indextemplate [$Album cget -indextemplate]
    prefs::addwidget "Page template file:" file \
	pagetemplate [$Album cget -pagetemplate]
}

# album::gui::saveprefs --
#
#	Configure the album with the selected preferences.

proc album::gui::saveprefs {preflist} {
    variable Album
    array set prefs $preflist
    $Album configure -indextemplate $prefs(indextemplate)
    $Album configure -pagetemplate $prefs(pagetemplate)
}


album::gui::init